@@ -1,6 +1,6 @@
 /* GStreamer
  *
- * Copyright (C) 2007 Sebastian Dröge <slomo@circular-chaos.org>
+ * Copyright (C) 2007,2010 Sebastian Dröge <sebastian.droege@collabora.co.uk>
  *
  * gstlfocontrolsource.c: Control source that provides some periodic waveforms
  *                        as control values.
@@ -43,7 +43,9 @@
 #include "gstlfocontrolsource.h"
 #include "gstlfocontrolsourceprivate.h"
 
-#include "math.h"
+#include "gst/glib-compat-private.h"
+
+#include <gst/math-compat.h>
 
 #define EMPTY(x) (x)
 
@@ -62,18 +64,13 @@ _calculate_pos (GstClockTime timestamp, GstClockTime timeshift,
 }
 
 #define DEFINE_SINE(type,round,convert) \
-\
 static inline g##type \
-_sine_get_##type (GstLFOControlSource *self, GstClockTime timestamp) \
+_sine_get_##type (GstLFOControlSource *self, g##type max, g##type min, gdouble amp, gdouble off, GstClockTime timeshift, GstClockTime period, gdouble frequency, GstClockTime timestamp) \
 { \
   gdouble ret; \
-  g##type max = g_value_get_##type (&self->priv->maximum_value); \
-  g##type min = g_value_get_##type (&self->priv->minimum_value); \
-  gdouble amp = convert (g_value_get_##type (&self->priv->amplitude)); \
-  gdouble off = convert (g_value_get_##type (&self->priv->offset)); \
-  GstClockTime pos = _calculate_pos (timestamp, self->priv->timeshift, self->priv->period); \
+  GstClockTime pos = _calculate_pos (timestamp, timeshift, period); \
   \
-  ret = sin (2.0 * M_PI * (self->priv->frequency / GST_SECOND) * gst_guint64_to_gdouble (pos)); \
+  ret = sin (2.0 * M_PI * (frequency / GST_SECOND) * gst_guint64_to_gdouble (pos)); \
   ret *= amp; \
   ret += off; \
   \
@@ -87,9 +84,20 @@ static gboolean \
 waveform_sine_get_##type (GstLFOControlSource *self, GstClockTime timestamp, \
     GValue *value) \
 { \
-  g##type ret; \
+  g##type ret, max, min; \
+  gdouble amp, off, frequency; \
+  GstClockTime timeshift, period; \
+  \
   g_mutex_lock (self->lock); \
-  ret = _sine_get_##type (self, timestamp); \
+  max = g_value_get_##type (&self->priv->maximum_value); \
+  min = g_value_get_##type (&self->priv->minimum_value); \
+  amp = convert (g_value_get_##type (&self->priv->amplitude)); \
+  off = convert (g_value_get_##type (&self->priv->offset)); \
+  timeshift = self->priv->timeshift; \
+  period = self->priv->period; \
+  frequency = self->priv->frequency; \
+  \
+  ret = _sine_get_##type (self, max, min, amp, off, timeshift, period, frequency, timestamp); \
   g_value_set_##type (value, ret); \
   g_mutex_unlock (self->lock); \
   return TRUE; \
@@ -102,10 +110,21 @@ waveform_sine_get_##type##_value_array (GstLFOControlSource *self, \
   gint i; \
   GstClockTime ts = timestamp; \
   g##type *values = (g##type *) value_array->values; \
+  g##type max, min; \
+  gdouble amp, off, frequency; \
+  GstClockTime timeshift, period; \
   \
   g_mutex_lock (self->lock); \
+  max = g_value_get_##type (&self->priv->maximum_value); \
+  min = g_value_get_##type (&self->priv->minimum_value); \
+  amp = convert (g_value_get_##type (&self->priv->amplitude)); \
+  off = convert (g_value_get_##type (&self->priv->offset)); \
+  timeshift = self->priv->timeshift; \
+  period = self->priv->period; \
+  frequency = self->priv->frequency; \
+  \
   for(i = 0; i < value_array->nbsamples; i++) { \
-    *values = _sine_get_##type (self, ts); \
+    *values = _sine_get_##type (self, max, min, amp, off, timeshift, period, frequency, ts); \
     ts += value_array->sample_interval; \
     values++; \
   } \
@@ -116,7 +135,6 @@ waveform_sine_get_##type##_value_array (GstLFOControlSource *self, \
 DEFINE_SINE (int, TRUE, EMPTY);
 DEFINE_SINE (uint, TRUE, EMPTY);
 DEFINE_SINE (long, TRUE, EMPTY);
-
 DEFINE_SINE (ulong, TRUE, EMPTY);
 DEFINE_SINE (int64, TRUE, EMPTY);
 DEFINE_SINE (uint64, TRUE, gst_guint64_to_gdouble);
@@ -145,14 +163,9 @@ static GstWaveformImplementation waveform_sine = {
 #define DEFINE_SQUARE(type,round, convert) \
 \
 static inline g##type \
-_square_get_##type (GstLFOControlSource *self, GstClockTime timestamp) \
+_square_get_##type (GstLFOControlSource *self, g##type max, g##type min, gdouble amp, gdouble off, GstClockTime timeshift, GstClockTime period, gdouble frequency, GstClockTime timestamp) \
 { \
-  g##type max = g_value_get_##type (&self->priv->maximum_value); \
-  g##type min = g_value_get_##type (&self->priv->minimum_value); \
-  gdouble amp = convert (g_value_get_##type (&self->priv->amplitude)); \
-  gdouble off = convert (g_value_get_##type (&self->priv->offset)); \
-  GstClockTime period = self->priv->period; \
-  GstClockTime pos = _calculate_pos (timestamp, self->priv->timeshift, period); \
+  GstClockTime pos = _calculate_pos (timestamp, timeshift, period); \
   gdouble ret; \
   \
   if (pos >= period / 2) \
@@ -172,9 +185,20 @@ static gboolean \
 waveform_square_get_##type (GstLFOControlSource *self, GstClockTime timestamp, \
     GValue *value) \
 { \
-  g##type ret; \
+  g##type ret, max, min; \
+  gdouble amp, off, frequency; \
+  GstClockTime timeshift, period; \
+  \
   g_mutex_lock (self->lock); \
-  ret = _square_get_##type (self, timestamp); \
+  max = g_value_get_##type (&self->priv->maximum_value); \
+  min = g_value_get_##type (&self->priv->minimum_value); \
+  amp = convert (g_value_get_##type (&self->priv->amplitude)); \
+  off = convert (g_value_get_##type (&self->priv->offset)); \
+  timeshift = self->priv->timeshift; \
+  period = self->priv->period; \
+  frequency = self->priv->frequency; \
+  \
+  ret = _square_get_##type (self, max, min, amp, off, timeshift, period, frequency, timestamp); \
   g_value_set_##type (value, ret); \
   g_mutex_unlock (self->lock); \
   return TRUE; \
@@ -187,10 +211,21 @@ waveform_square_get_##type##_value_array (GstLFOControlSource *self, \
   gint i; \
   GstClockTime ts = timestamp; \
   g##type *values = (g##type *) value_array->values; \
+  g##type max, min; \
+  gdouble amp, off, frequency; \
+  GstClockTime timeshift, period; \
   \
   g_mutex_lock (self->lock); \
+  max = g_value_get_##type (&self->priv->maximum_value); \
+  min = g_value_get_##type (&self->priv->minimum_value); \
+  amp = convert (g_value_get_##type (&self->priv->amplitude)); \
+  off = convert (g_value_get_##type (&self->priv->offset)); \
+  timeshift = self->priv->timeshift; \
+  period = self->priv->period; \
+  frequency = self->priv->frequency; \
+  \
   for(i = 0; i < value_array->nbsamples; i++) { \
-    *values = _square_get_##type (self, ts); \
+    *values = _square_get_##type (self, max, min, amp, off, timeshift, period, frequency, ts); \
     ts += value_array->sample_interval; \
     values++; \
   } \
@@ -199,10 +234,8 @@ waveform_square_get_##type##_value_array (GstLFOControlSource *self, \
 }
 
 DEFINE_SQUARE (int, TRUE, EMPTY);
-
 DEFINE_SQUARE (uint, TRUE, EMPTY);
 DEFINE_SQUARE (long, TRUE, EMPTY);
-
 DEFINE_SQUARE (ulong, TRUE, EMPTY);
 DEFINE_SQUARE (int64, TRUE, EMPTY);
 DEFINE_SQUARE (uint64, TRUE, gst_guint64_to_gdouble);
@@ -231,14 +264,9 @@ static GstWaveformImplementation waveform_square = {
 #define DEFINE_SAW(type,round,convert) \
 \
 static inline g##type \
-_saw_get_##type (GstLFOControlSource *self, GstClockTime timestamp) \
+_saw_get_##type (GstLFOControlSource *self, g##type max, g##type min, gdouble amp, gdouble off, GstClockTime timeshift, GstClockTime period, gdouble frequency, GstClockTime timestamp) \
 { \
-  g##type max = g_value_get_##type (&self->priv->maximum_value); \
-  g##type min = g_value_get_##type (&self->priv->minimum_value); \
-  gdouble amp = convert (g_value_get_##type (&self->priv->amplitude)); \
-  gdouble off = convert (g_value_get_##type (&self->priv->offset)); \
-  GstClockTime period = self->priv->period; \
-  GstClockTime pos = _calculate_pos (timestamp, self->priv->timeshift, period); \
+  GstClockTime pos = _calculate_pos (timestamp, timeshift, period); \
   gdouble ret; \
   \
   ret = - ((gst_guint64_to_gdouble (pos) - gst_guint64_to_gdouble (period) / 2.0) * ((2.0 * amp) / gst_guint64_to_gdouble (period)));\
@@ -255,9 +283,20 @@ static gboolean \
 waveform_saw_get_##type (GstLFOControlSource *self, GstClockTime timestamp, \
     GValue *value) \
 { \
-  g##type ret; \
+  g##type ret, max, min; \
+  gdouble amp, off, frequency; \
+  GstClockTime timeshift, period; \
+  \
   g_mutex_lock (self->lock); \
-  ret = _saw_get_##type (self, timestamp); \
+  max = g_value_get_##type (&self->priv->maximum_value); \
+  min = g_value_get_##type (&self->priv->minimum_value); \
+  amp = convert (g_value_get_##type (&self->priv->amplitude)); \
+  off = convert (g_value_get_##type (&self->priv->offset)); \
+  timeshift = self->priv->timeshift; \
+  period = self->priv->period; \
+  frequency = self->priv->frequency; \
+  \
+  ret = _saw_get_##type (self, max, min, amp, off, timeshift, period, frequency, timestamp); \
   g_value_set_##type (value, ret); \
   g_mutex_unlock (self->lock); \
   return TRUE; \
@@ -270,10 +309,21 @@ waveform_saw_get_##type##_value_array (GstLFOControlSource *self, \
   gint i; \
   GstClockTime ts = timestamp; \
   g##type *values = (g##type *) value_array->values; \
+  g##type max, min; \
+  gdouble amp, off, frequency; \
+  GstClockTime timeshift, period; \
   \
   g_mutex_lock (self->lock); \
+  max = g_value_get_##type (&self->priv->maximum_value); \
+  min = g_value_get_##type (&self->priv->minimum_value); \
+  amp = convert (g_value_get_##type (&self->priv->amplitude)); \
+  off = convert (g_value_get_##type (&self->priv->offset)); \
+  timeshift = self->priv->timeshift; \
+  period = self->priv->period; \
+  frequency = self->priv->frequency; \
+  \
   for(i = 0; i < value_array->nbsamples; i++) { \
-    *values = _saw_get_##type (self, ts); \
+    *values = _saw_get_##type (self, max, min, amp, off, timeshift, period, frequency, ts); \
     ts += value_array->sample_interval; \
     values++; \
   } \
@@ -282,10 +332,8 @@ waveform_saw_get_##type##_value_array (GstLFOControlSource *self, \
 }
 
 DEFINE_SAW (int, TRUE, EMPTY);
-
 DEFINE_SAW (uint, TRUE, EMPTY);
 DEFINE_SAW (long, TRUE, EMPTY);
-
 DEFINE_SAW (ulong, TRUE, EMPTY);
 DEFINE_SAW (int64, TRUE, EMPTY);
 DEFINE_SAW (uint64, TRUE, gst_guint64_to_gdouble);
@@ -314,14 +362,9 @@ static GstWaveformImplementation waveform_saw = {
 #define DEFINE_RSAW(type,round,convert) \
 \
 static inline g##type \
-_rsaw_get_##type (GstLFOControlSource *self, GstClockTime timestamp) \
+_rsaw_get_##type (GstLFOControlSource *self, g##type max, g##type min, gdouble amp, gdouble off, GstClockTime timeshift, GstClockTime period, gdouble frequency, GstClockTime timestamp) \
 { \
-  g##type max = g_value_get_##type (&self->priv->maximum_value); \
-  g##type min = g_value_get_##type (&self->priv->minimum_value); \
-  gdouble amp = convert (g_value_get_##type (&self->priv->amplitude)); \
-  gdouble off = convert (g_value_get_##type (&self->priv->offset)); \
-  GstClockTime period = self->priv->period; \
-  GstClockTime pos = _calculate_pos (timestamp, self->priv->timeshift, period); \
+  GstClockTime pos = _calculate_pos (timestamp, timeshift, period); \
   gdouble ret; \
   \
   ret = ((gst_guint64_to_gdouble (pos) - gst_guint64_to_gdouble (period) / 2.0) * ((2.0 * amp) / gst_guint64_to_gdouble (period)));\
@@ -338,9 +381,20 @@ static gboolean \
 waveform_rsaw_get_##type (GstLFOControlSource *self, GstClockTime timestamp, \
     GValue *value) \
 { \
-  g##type ret; \
+  g##type ret, max, min; \
+  gdouble amp, off, frequency; \
+  GstClockTime timeshift, period; \
+  \
   g_mutex_lock (self->lock); \
-  ret = _rsaw_get_##type (self, timestamp); \
+  max = g_value_get_##type (&self->priv->maximum_value); \
+  min = g_value_get_##type (&self->priv->minimum_value); \
+  amp = convert (g_value_get_##type (&self->priv->amplitude)); \
+  off = convert (g_value_get_##type (&self->priv->offset)); \
+  timeshift = self->priv->timeshift; \
+  period = self->priv->period; \
+  frequency = self->priv->frequency; \
+  \
+  ret = _rsaw_get_##type (self, max, min, amp, off, timeshift, period, frequency, timestamp); \
   g_value_set_##type (value, ret); \
   g_mutex_unlock (self->lock); \
   return TRUE; \
@@ -353,10 +407,21 @@ waveform_rsaw_get_##type##_value_array (GstLFOControlSource *self, \
   gint i; \
   GstClockTime ts = timestamp; \
   g##type *values = (g##type *) value_array->values; \
+  g##type max, min; \
+  gdouble amp, off, frequency; \
+  GstClockTime timeshift, period; \
   \
   g_mutex_lock (self->lock); \
+  max = g_value_get_##type (&self->priv->maximum_value); \
+  min = g_value_get_##type (&self->priv->minimum_value); \
+  amp = convert (g_value_get_##type (&self->priv->amplitude)); \
+  off = convert (g_value_get_##type (&self->priv->offset)); \
+  timeshift = self->priv->timeshift; \
+  period = self->priv->period; \
+  frequency = self->priv->frequency; \
+  \
   for(i = 0; i < value_array->nbsamples; i++) { \
-    *values = _rsaw_get_##type (self, ts); \
+    *values = _rsaw_get_##type (self, max, min, amp, off, timeshift, period, frequency, ts); \
     ts += value_array->sample_interval; \
     values++; \
   } \
@@ -365,10 +430,8 @@ waveform_rsaw_get_##type##_value_array (GstLFOControlSource *self, \
 }
 
 DEFINE_RSAW (int, TRUE, EMPTY);
-
 DEFINE_RSAW (uint, TRUE, EMPTY);
 DEFINE_RSAW (long, TRUE, EMPTY);
-
 DEFINE_RSAW (ulong, TRUE, EMPTY);
 DEFINE_RSAW (int64, TRUE, EMPTY);
 DEFINE_RSAW (uint64, TRUE, gst_guint64_to_gdouble);
@@ -397,14 +460,9 @@ static GstWaveformImplementation waveform_rsaw = {
 #define DEFINE_TRIANGLE(type,round,convert) \
 \
 static inline g##type \
-_triangle_get_##type (GstLFOControlSource *self, GstClockTime timestamp) \
+_triangle_get_##type (GstLFOControlSource *self, g##type max, g##type min, gdouble amp, gdouble off, GstClockTime timeshift, GstClockTime period, gdouble frequency, GstClockTime timestamp) \
 { \
-  g##type max = g_value_get_##type (&self->priv->maximum_value); \
-  g##type min = g_value_get_##type (&self->priv->minimum_value); \
-  gdouble amp = convert (g_value_get_##type (&self->priv->amplitude)); \
-  gdouble off = convert (g_value_get_##type (&self->priv->offset)); \
-  GstClockTime period = self->priv->period; \
-  GstClockTime pos = _calculate_pos (timestamp, self->priv->timeshift, period); \
+  GstClockTime pos = _calculate_pos (timestamp, timeshift, period); \
   gdouble ret; \
   \
   if (gst_guint64_to_gdouble (pos) <= gst_guint64_to_gdouble (period) / 4.0) \
@@ -426,9 +484,20 @@ static gboolean \
 waveform_triangle_get_##type (GstLFOControlSource *self, GstClockTime timestamp, \
     GValue *value) \
 { \
-  g##type ret; \
+  g##type ret, max, min; \
+  gdouble amp, off, frequency; \
+  GstClockTime timeshift, period; \
+  \
   g_mutex_lock (self->lock); \
-  ret = _triangle_get_##type (self, timestamp); \
+  max = g_value_get_##type (&self->priv->maximum_value); \
+  min = g_value_get_##type (&self->priv->minimum_value); \
+  amp = convert (g_value_get_##type (&self->priv->amplitude)); \
+  off = convert (g_value_get_##type (&self->priv->offset)); \
+  timeshift = self->priv->timeshift; \
+  period = self->priv->period; \
+  frequency = self->priv->frequency; \
+  \
+  ret = _triangle_get_##type (self, max, min, amp, off, timeshift, period, frequency, timestamp); \
   g_value_set_##type (value, ret); \
   g_mutex_unlock (self->lock); \
   return TRUE; \
@@ -441,10 +510,21 @@ waveform_triangle_get_##type##_value_array (GstLFOControlSource *self, \
   gint i; \
   GstClockTime ts = timestamp; \
   g##type *values = (g##type *) value_array->values; \
+  g##type max, min; \
+  gdouble amp, off, frequency; \
+  GstClockTime timeshift, period; \
   \
   g_mutex_lock (self->lock); \
+  max = g_value_get_##type (&self->priv->maximum_value); \
+  min = g_value_get_##type (&self->priv->minimum_value); \
+  amp = convert (g_value_get_##type (&self->priv->amplitude)); \
+  off = convert (g_value_get_##type (&self->priv->offset)); \
+  timeshift = self->priv->timeshift; \
+  period = self->priv->period; \
+  frequency = self->priv->frequency; \
+  \
   for(i = 0; i < value_array->nbsamples; i++) { \
-    *values = _triangle_get_##type (self, ts); \
+    *values = _triangle_get_##type (self, max, min, amp, off, timeshift, period, frequency, ts); \
     ts += value_array->sample_interval; \
     values++; \
   } \
@@ -453,10 +533,8 @@ waveform_triangle_get_##type##_value_array (GstLFOControlSource *self, \
 }
 
 DEFINE_TRIANGLE (int, TRUE, EMPTY);
-
 DEFINE_TRIANGLE (uint, TRUE, EMPTY);
 DEFINE_TRIANGLE (long, TRUE, EMPTY);
-
 DEFINE_TRIANGLE (ulong, TRUE, EMPTY);
 DEFINE_TRIANGLE (int64, TRUE, EMPTY);
 DEFINE_TRIANGLE (uint64, TRUE, gst_guint64_to_gdouble);
@@ -490,7 +568,7 @@ static GstWaveformImplementation *waveforms[] = {
   &waveform_triangle
 };
 
-static guint num_waveforms = G_N_ELEMENTS (waveforms);
+static const guint num_waveforms = G_N_ELEMENTS (waveforms);
 
 enum
 {
@@ -504,31 +582,28 @@ enum
 GType
 gst_lfo_waveform_get_type (void)
 {
-  static GType gtype = 0;
+  static gsize gtype = 0;
+  static const GEnumValue values[] = {
+    {GST_LFO_WAVEFORM_SINE, "GST_LFO_WAVEFORM_SINE",
+        "sine"},
+    {GST_LFO_WAVEFORM_SQUARE, "GST_LFO_WAVEFORM_SQUARE",
+        "square"},
+    {GST_LFO_WAVEFORM_SAW, "GST_LFO_WAVEFORM_SAW",
+        "saw"},
+    {GST_LFO_WAVEFORM_REVERSE_SAW, "GST_LFO_WAVEFORM_REVERSE_SAW",
+        "reverse-saw"},
+    {GST_LFO_WAVEFORM_TRIANGLE, "GST_LFO_WAVEFORM_TRIANGLE",
+        "triangle"},
+    {0, NULL, NULL}
+  };
 
-  if (gtype == 0) {
-    static const GEnumValue values[] = {
-      {GST_LFO_WAVEFORM_SINE, "Sine waveform (default)",
-          "sine"},
-      {GST_LFO_WAVEFORM_SQUARE, "Square waveform",
-          "square"},
-      {GST_LFO_WAVEFORM_SAW, "Saw waveform",
-          "saw"},
-      {GST_LFO_WAVEFORM_REVERSE_SAW, "Reverse saw waveform",
-          "reverse-saw"},
-      {GST_LFO_WAVEFORM_TRIANGLE, "Triangle waveform",
-          "triangle"},
-      {0, NULL, NULL}
-    };
-
-    gtype = g_enum_register_static ("GstLFOWaveform", values);
+  if (g_once_init_enter (&gtype)) {
+    GType tmp = g_enum_register_static ("GstLFOWaveform", values);
+    g_once_init_leave (&gtype, tmp);
   }
-  return gtype;
-}
 
-static void gst_lfo_control_source_init (GstLFOControlSource * self);
-static void
-gst_lfo_control_source_class_init (GstLFOControlSourceClass * klass);
+  return (GType) gtype;
+}
 
 G_DEFINE_TYPE (GstLFOControlSource, gst_lfo_control_source,
     GST_TYPE_CONTROL_SOURCE);
@@ -566,7 +641,7 @@ gst_lfo_control_source_reset (GstLFOControlSource * self)
 GstLFOControlSource *
 gst_lfo_control_source_new (void)
 {
-  return g_object_new (GST_TYPE_LFO_CONTROL_SOURCE, NULL);
+  return g_object_newv (GST_TYPE_LFO_CONTROL_SOURCE, 0, NULL);
 }
 
 static gboolean
@@ -576,7 +651,7 @@ gst_lfo_control_source_set_waveform (GstLFOControlSource * self,
   GstControlSource *csource = GST_CONTROL_SOURCE (self);
   gboolean ret = TRUE;
 
-  if (waveform >= num_waveforms || waveform < 0) {
+  if (waveform >= num_waveforms || (int) waveform < 0) {
     GST_WARNING ("waveform %d invalid or not implemented yet", waveform);
     return FALSE;
   }
@@ -794,12 +869,12 @@ gst_lfo_control_source_bind (GstControlSource * source, GParamSpec * pspec)
       g_value_set_double (&self->priv->maximum_value, tpspec->maximum);
       if (!G_IS_VALUE (&self->priv->amplitude)) {
         g_value_init (&self->priv->amplitude, type);
-        g_value_set_float (&self->priv->amplitude, 0.0);
+        g_value_set_double (&self->priv->amplitude, 0.0);
       }
 
       if (!G_IS_VALUE (&self->priv->offset)) {
         g_value_init (&self->priv->offset, type);
-        g_value_set_float (&self->priv->offset, tpspec->default_value);
+        g_value_set_double (&self->priv->offset, tpspec->default_value);
       }
       break;
     }
@@ -896,7 +971,8 @@ gst_lfo_control_source_set_property (GObject * object, guint prop_id,
   switch (prop_id) {
     case PROP_WAVEFORM:
       g_mutex_lock (self->lock);
-      gst_lfo_control_source_set_waveform (self, g_value_get_enum (value));
+      gst_lfo_control_source_set_waveform (self,
+          (GstLFOWaveform) g_value_get_enum (value));
       g_mutex_unlock (self->lock);
       break;
     case PROP_FREQUENCY:{
