@@ -48,6 +48,7 @@
 #endif
 
 #include "gsttee.h"
+#include "gst/glib-compat-private.h"
 
 #include <string.h>
 
@@ -551,7 +552,9 @@ gst_tee_buffer_alloc (GstPad * pad, guint64 offset, guint size,
   GstFlowReturn res;
   GstPad *allocpad;
 
-  tee = GST_TEE_CAST (GST_PAD_PARENT (pad));
+  tee = GST_TEE_CAST (gst_pad_get_parent (pad));
+  if (G_UNLIKELY (tee == NULL))
+    return GST_FLOW_WRONG_STATE;
 
   res = GST_FLOW_NOT_LINKED;
 
@@ -578,7 +581,7 @@ gst_tee_buffer_alloc (GstPad * pad, guint64 offset, guint size,
 
     GST_OBJECT_LOCK (tee);
   }
-  /* either we failed to alloc on the the previous pad or we did not have a
+  /* either we failed to alloc on the previous pad or we did not have a
    * previous pad. */
   if (res == GST_FLOW_NOT_LINKED) {
     /* find a new pad to alloc a buffer on */
@@ -587,6 +590,7 @@ gst_tee_buffer_alloc (GstPad * pad, guint64 offset, guint size,
   }
   GST_OBJECT_UNLOCK (tee);
 
+  gst_object_unref (tee);
   return res;
 }
 
