@@ -45,7 +45,7 @@ cleanup (void)
 }
 
 static GstBuffer *
-buffer_from_string (gchar * str)
+buffer_from_string (const gchar * str)
 {
   guint size;
   GstBuffer *buf;
@@ -472,7 +472,7 @@ GST_START_TEST (test_do)
           gst_buffer_list_iterator_do (it,
               (GstBufferListDoFunction) gst_buffer_ref, NULL)));
   fail_unless (buf == NULL);
-  data = "data";
+  data = (char *) "data";
   ASSERT_CRITICAL ((buf = gst_buffer_list_iterator_do (it, do_data_func,
               data)));
   fail_unless (buf == NULL);
@@ -766,6 +766,43 @@ GST_START_TEST (test_foreach)
 
 GST_END_TEST;
 
+GST_START_TEST (test_list)
+{
+  GstBufferListIterator *it;
+  GList *l = NULL;
+  gint i;
+
+  for (i = 0; i < 10; i++) {
+    gchar name[10];
+    g_snprintf (name, 10, "%d", i);
+    l = g_list_append (l, buffer_from_string (name));
+  }
+
+  /* add buffers to the list */
+  it = gst_buffer_list_iterate (list);
+  gst_buffer_list_iterator_add_group (it);
+  gst_buffer_list_iterator_add_list (it, l);
+
+  /* add a buffer */
+  gst_buffer_list_iterator_add (it, buffer_from_string ("10"));
+
+  /* add another list */
+  l = g_list_append (NULL, buffer_from_string ("11"));
+  gst_buffer_list_iterator_add_list (it, l);
+
+  for (i = 0; i < 12; i++) {
+    GstBuffer *buf;
+    gchar name[10];
+
+    buf = gst_buffer_list_get (list, 0, i);
+    g_snprintf (name, 10, "%d", i);
+    fail_unless (memcmp (name, (gchar *) GST_BUFFER_DATA (buf),
+            GST_BUFFER_SIZE (buf)) == 0);
+  }
+  gst_buffer_list_iterator_free (it);
+}
+
+GST_END_TEST;
 
 static Suite *
 gst_buffer_list_suite (void)
@@ -783,6 +820,7 @@ gst_buffer_list_suite (void)
   tcase_add_test (tc_chain, test_do);
   tcase_add_test (tc_chain, test_merge);
   tcase_add_test (tc_chain, test_foreach);
+  tcase_add_test (tc_chain, test_list);
 
   return s;
 }
